--
-- RocketLauncher
-- by CraigRock, John & Jozef
--

RocketLauncher = {};


function RocketLauncher.prerequisitesPresent(specializations)
    return true;
end;

function RocketLauncher:load(xmlFile)
    self.onRocketCollided = DeviceSpecializationUtil.callSpecializationsFunction("onRocketCollided");
    self.overlapRocketLauncherCallback = DeviceSpecializationUtil.callSpecializationsFunction("overlapRocketLauncherCallback");


    self.isVibrating = false;


    self.rocketExplodeParticleSystemData = {};
    Utils.loadParticleSystemData(xmlFile, self.rocketExplodeParticleSystemData, "device.rocketExplodeParticleSystem"); --, self.deviceNode, false, nil, self.baseDirectory);
    self.rocketExplodeParticleSystemDisableDuration = Utils.getNoNil(getXMLFloat(xmlFile, "device.rocketExplodeParticleSystem#disableDuration"), 1)*1000;
    self.rocketExplodeParticleSystemDeleteDuration = Utils.getNoNil(getXMLFloat(xmlFile, "device.rocketExplodeParticleSystem#deleteDuration"), 10*self.rocketExplodeParticleSystemDisableDuration)*1000;

    self.rocketExplodeParticleSystems = {};

    self.rocketExplodeSoundFile = getXMLString(xmlFile, "device.rocketExplodeSound#file");
    if self.rocketExplodeSoundFile ~= nil then
        self.rocketExplodeSoundFile = Utils.getFilename(self.rocketExplodeSoundFile, self.baseDirectory);
    end;
    self.rocketExplodeSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "device.rocketExplodeSound#pitchOffset"), 1);
    self.rocketExplodeSoundRadius = Utils.getNoNil(getXMLFloat(xmlFile, "device.rocketExplodeSound#radius"), 50);
    self.rocketExplodeSoundInnerRadius = Utils.getNoNil(getXMLFloat(xmlFile, "device.rocketExplodeSound#innerRadius"), 10);
    self.rocketExplodeSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "device.rocketExplodeSound#volume"), 1);


    self.rocketExplodeRadius = Utils.getNoNil(getXMLFloat(xmlFile, "device.rocket#explodeDamage"), 1);
    self.rocketExplodeDamage = Utils.getNoNil(getXMLFloat(xmlFile, "device.rocket#explodeForce"), 1)*0.001;
    self.rocketExplodeForce = Utils.getNoNil(getXMLFloat(xmlFile, "device.rocket#explodeRadius"), 0.5);

    self.rocketFilename = getXMLString(xmlFile, "device.rocket#filename");
    self.rocketNode = Utils.indexToObject(self.deviceNode, getXMLString(xmlFile, "device.rocket#node"))
    if self.rocketNode ~= nil then
        setVisibility(self.rocketNode, true);
    end;
    self.rocketStartNode = Utils.indexToObject(self.deviceNode, getXMLString(xmlFile, "device.rocket#startNode"));
    if self.rocketStartNode == nil then
        self.rocketStartNode = self.deviceNode;
    end;
    self.rocketVelocity = Utils.getNoNil(getXMLFloat(xmlFile, "device.rocket#velocity"), 30)*0.001;
    self.rocketGravity = Utils.getNoNil(getXMLFloat(xmlFile, "device.rocket#gravity"), 0.001)*0.001;

    self.rocketExplodeSounds = {};

    self.rockets = {};

    self.crossHairSize = 0.035;
    self.crossHairSizeHalf = self.crossHairSize/2;
    self.crossHairId = createImageOverlay(Utils.getFilename("$data/devices/sledgehammer/sledgehammerCrosshair.png", self.baseDirectory));

    self.shotAnimation = getAnimCharacterSet(getChildAt(self.deviceNode, 0));
    if self.shotAnimation ~= nil then
        self.shotAnimationDuration = getAnimClipDuration(self.shotAnimation, 0);
        assignAnimTrackClip(self.shotAnimation, 0, 0);
        setAnimTrackBlendWeight(self.shotAnimation, 0, 1.0);
        setAnimTrackTime(self.shotAnimation, 0, self.shotAnimationDuration);
        setAnimTrackSpeedScale(self.shotAnimation, 0, 1);
        setAnimTrackLoopState(self.shotAnimation, 0, false);
    end;

    self.needsInactiveUpdate = true;
end;

function RocketLauncher:delete()
    delete(self.crossHairId);
    for _,v in pairs(self.rocketExplodeParticleSystems) do
        Utils.deleteParticleSystem(v);
    end;
    for _,v in pairs(self.rocketExplodeSounds) do
        delete(v.audioSource);
    end;
    RocketLauncher.disableVibration(self);
end;

function RocketLauncher:disableVibration()
    if (self.isVibrating) then
        self.isVibrating = false;
        local gamepadDeviceId = InputBinding.getGamepadDeviceIdOfDigitalAction(InputBinding.ACTIVATE_HANDTOOL);
        if (gamepadDeviceId) then
            setGamepadVibration(0.0, 0.0, -1, gamepadDeviceId);
        end;
    end;
end;

function RocketLauncher:update(dt)
    if self:getIsActive() then
        if self.shotAnimation == nil or getAnimTrackTime(self.shotAnimation, 0) >= self.shotAnimationDuration then
            if self:getIsActiveForInput() and InputBinding.hasEvent(InputBinding.ACTIVATE_HANDTOOL) and (not self.takeInHandActive) then
				
				playSample(self.idleSound, 1, 1, 0);
                
				setAnimTrackTime(self.shotAnimation, 0, 0.0);
                enableAnimTrack(self.shotAnimation, 0);
                local x,y,z = getWorldTranslation(self.rocketStartNode);
                local dx,dy,dz = localDirectionToWorld(self.rocketStartNode, 0,0,1);
                local rocket = Rocket:new(self, self.rocketFilename, self.baseDirectory, x,y,z, dx,dy,dz, self.rocketVelocity, self.rocketGravity);
                if rocket ~= nil then
                    g_currentMission:addUpdateable(rocket);
                end;
            end;
        end;
    end;

    for k,v in pairs(self.rocketExplodeParticleSystems) do
        if g_currentMission.physicsPaused then
            v.deleteTime = v.deleteTime+dt;
            v.disableTime = v.disableTime+dt;
        end;
        if v.deleteTime <= self.time then
            Utils.deleteParticleSystem(v);
            self.rocketExplodeParticleSystems[k] = nil;
        elseif v.disableTime <= self.time then
            Utils.setEmittingState(v, false);
            v.disableTime = 24*60*60*1000;
        end;
    end;
    for k,v in pairs(self.rocketExplodeSounds) do
        if v.deleteTime <= self.time then
            delete(v.audioSource);
            self.rocketExplodeSounds[k] = nil;
        end;
    end;
end;

function RocketLauncher:draw()
    renderOverlay(self.crossHairId, 0.5-self.crossHairSizeHalf*0.75 + 0.01, 0.5-self.crossHairSizeHalf - 0.005, self.crossHairSize*0.75, self.crossHairSize);
end;

function RocketLauncher:onActivate()
end;

function RocketLauncher:onDeactivate()
end;

function RocketLauncher:onRocketCollided(rocket, x, y, z)

	playSample(self.hitSound, 1, 1, 0);


    --g_currentMission:removeUpdateable(rocket);
    self.rocketPositionX = x;
    self.rocketPositionY = y;
    self.rocketPositionZ = z;
    overlapSphere(x, y, z, self.rocketExplodeRadius, "overlapRocketLauncherCallback", self);


    local particleSystem = {};
    particleSystem.disableTime = self.time + self.rocketExplodeParticleSystemDisableDuration;
    particleSystem.deleteTime = self.time + self.rocketExplodeParticleSystemDeleteDuration;
    self.rocketExplodeParticleSystemData.posX = x;
    self.rocketExplodeParticleSystemData.posY = y;
    self.rocketExplodeParticleSystemData.posZ = z;
    Utils.loadParticleSystemFromData(self.rocketExplodeParticleSystemData, particleSystem, getRootNode(), true, nil, self.baseDirectory);
    self.rocketExplodeParticleSystems[particleSystem] = particleSystem;
    Utils.setEmittingState(particleSystem, true);

    if self.rocketExplodeSoundFile ~= nil then
        local explodeSound = {};
        explodeSound.audioSource = createAudioSource("explodeAudioSource", self.rocketExplodeSoundFile, self.rocketExplodeSoundRadius, self.rocketExplodeSoundInnerRadius, self.rocketExplodeSoundVolume, 1);
        link(getRootNode(), explodeSound.audioSource);
        setTranslation(explodeSound.audioSource, x, y, z);
        local sample = getAudioSourceSample(explodeSound.audioSource);
        if sample == 0 then
            delete(explodeSound.audioSource);
        else
            setSamplePitch(sample, self.rocketExplodeSoundPitchOffset);
            explodeSound.deleteTime = self.time + 100+getSampleDuration(sample);
            self.rocketExplodeSounds[explodeSound] = explodeSound;
        end;
    end;
end;

function RocketLauncher:overlapRocketLauncherCallback(transformId, subShapeIndex)
    if subShapeIndex >= 0 and getHasClassId(transformId, ClassIds.DESTRUCTION_SHAPE) then
        local x = self.rocketPositionX;
        local y = self.rocketPositionY;
        local z = self.rocketPositionZ;
        local mx, my, mz = getDestructionShapeCenterOfMass(transformId, subShapeIndex);
        local tx, ty, tz = destructionShapeLocalToWorld(transformId, subShapeIndex, true, mx, my, mz);
        local dx = tx-x;
        local dy = ty-y;
        local dz = tz-z;
        local len = Utils.vector3Length(dx, dy, dz);
        local scale = 10*getDestructionShapeMass(transformId, subShapeIndex);
        if len < 0.0001 then
            dx = 0;
            dy = 0.01*scale;
            dz = 0;
        else
            dx = dx * scale / len;
            dy = dy * scale / len
            dz = dz * scale / len;
        end;
        addDestructionShapeImpulse(transformId, subShapeIndex, dx*self.rocketExplodeForce, dy*self.rocketExplodeForce, dz*self.rocketExplodeForce, tx, ty, tz, false);

        addDestructionShapeDamage(transformId, subShapeIndex, self.rocketExplodeDamage*self.rocketExplodeRadius/len);
    end;
end;
